; GRABTIM3.ASM  --  ATTINY2313 micro-controller project
;   Uses new direct drive for relay (no capacitor)

; ***** / Grabs time, stores it and executes commands repeatedly

; ***** / 3.579545 MHz Xtl for clock

; October 12, 2006 -  EEPROM save enabled this date
;     Button3 button-push routine replaced old DetectCnt routine

.include "tn2313def.inc"

; R16 general purpose temporary use register
; R20 contains downcnt.
; R21 contains TPCounter count


;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt

	
START1:			;set up stack pointer and initialize cold start

ldi 	r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out	SPL,R16			; Initialize stack pointer to end of internal RAM

ser	r16		;Set to all ones for out port direction as output
out	DDRB,r16	;Set PORTB to output

clr	r16
out	PortB,r16	;LEDs are off when port pins are low

ldi	R16,254		;Set bit 0 as input ... remaining bits as output
out	DDRD,R16	;execute data direction with bit 0 as input
out	PORTD,R16	;execute to set out ports high, and in port with
			; no input resistor

in	r16,TIMSK	;enable 16-bit match to A register
ori	r16,64		
out	TIMSK,r16

in	r16,TCCR1B
ori	r16,11		; divide by 64 on prescaler
out	TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
			;and bits 0,1,2 select prescaling source as 64 of clock

ldi	r16,218
out	OCR1AH,r16
ldi	r16,126		;default selected at 126 for this particular crystal
out	OCR1AL,r16	;loads the compare register A for timer 1 (1 second)
			; -- clock adj. to run slow (for Corr Purp)
			
rcall	ZeroCommandMemory		;zero all Command Memory locations


clr	r18		;zero all flags in register r18


InitializeClkTime:		;starts clock at 8:00 a.m.

ldi	r16,1
sts	$63,r16			;put Sunday(1) in as start of day-of-week

ldi	R16,224			;8:00 A.M. low byte
sts	$61,r16		

ldi	r16,1			;8:00 A.M. high byte
sts	$62,R16

mov	r15,r16			;set seconds counter to one

clr	r16
sts	$64,r16			 ;default load of clock correction - add 0 sec.
	
sts	$65,r16			; permanent - default = 0 = no correction 
sts	$66,r16			; variable - 0= no correction, 1 = every day
					; 2 = every other day, 3 = every 3rd day, etc.
					; default = add 0 second each day

ldi	r26,$006A		;initialize at start of memory locations
				
ldi	r16,$6C			;set up start of recording area for data
sts	$67,r16			;bookmark start of command memory

ldi	r16,3			;set default of variable day to three
sts	$69,r16
sts	$6A,r16
dec	r16				;set every-other-day value into memory (2)
sts	$68,r16

bset	7			;enable global interrupts


;------------------------------------


MainLoop:

sbic	PIND,0x00	;skip if pin 0, port D switch down
rjmp	ML1a

rcall	DetectCntTwo


ML1a:

mov		r16,r15			;check for 00 of seconds counter
cpi		r16,00
breq	ML1

cpi		r16,30			;check for 30 seconds into count
breq	ML3b
rjmp	ML4


ML3b:

sbrs	r18,5		;skip if bit 5(32) every-other-day and variable day
			; flag is set
rjmp	ML3


ML3a:

rcall	DisplayDayCountdown	;display the day countdown

sbrs	r18,3			;skip if grab time bit 3(8) is set
rjmp	ML4


ML3k:

ldi		r16,16
rcall	RapidRedFlashInterruptible	;flash red led rapidly 16 times
rcall	Idelay2310
					; to indicate record mode in effect
rjmp	ML4


ML3:


ldi		r16,6
rjmp	ML1b


ML1:

ldi		r16,16


ML1b:

sbrc	r18,3		;skip if bit 3 (8), grabtime enable is disabled
rjmp	ML1d


rcall	VarWagRGinterruptible


ML1c:

rcall	DisplayClock


ML4:

rjmp	MainLoop	;circle back to repeat loop


ML1d:			;GrabTime is enabled -- flash out rapid red flashes

rcall	RapidRedFlashInterruptible
rjmp	ML1c


;---------------------------

Button3:		;Replaces old DetectCnt  2/15/2006

push	r23

clr		r20		;HoldDown count
clr		r21		;short touch count 1-9
clr		r23		;clear countdown register


B1:

inc		r23
rcall	Delay40th
cpi		r23,80		;delay before first holddown count - 40=one second - 3 seconds
brsh	B3			;branch if - it is a holddown count

sbis	PIND,0		;skip if switch is NOT down
rjmp	B1


B2b:				;Count is detected

inc		r21			;step short count one count - a dit short touch command	
rcall	Delay40th
clr		r23			;clear r23


B2d:

sbis	PIND,0		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,80		;has switch been up 80 counts? 40=one second (3 seconds)
brsh	B5

rjmp	B2d


B2e:

clr		r23
rjmp	B1


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi     PortB,0
rcall   Delay40th
cbi     PortB,0         ;flash PROGRAM LED


clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,0		;skip if switch is down
rjmp	B4a

cpi		r23,20
brlo	B4

rjmp	B3


B4a:

rcall	Delay40th


B5:

pop		r23
ret


;------------------------

Button3Delay:

sbic	PIND,0		;skip if switch is down
rjmp	Button3Delay
rjmp	Button3


;------------------------

DetectCntTwo:		;includes string of functions requiring attention

push	r16

rcall	Button3	
	
			;END of Button3 --- now comes the decoding

cpi		r20,1		;checks to see if LONG Count has been recorded
brsh	LongCount	;branch if same or higher than 1

; The following are short counts


ShortCount:

sbrc	r18,3		;skip if bit 3 (8), grab time clear
rjmp	SC2		;SC1 one not available when grab time on


SC1:

cpi		r21,1		;#1 short = Togle AC on/off
breq	SC1a
rjmp	SC1Out

SC1a:

rcall	ToggleAC
rjmp	LCCEnd


;-----------------------------


SC1Out:			;jumps out if grab flag is not set

sbrc	r18,3		;skip if grab time bit 3 flag is clear

rjmp	SC2		;It is in grab time mode so continue further SC detection

rjmp	LCCEnd		;It is in PLAY mode so end all detection of other counts


;-----------------------------

SC2:			;two short depresses store every day time command

cpi		r21,2		;#1 short = Togle AC on/off
breq	SC2a
rjmp	SC3

SC2a:

clr		r16
sts		$6B,r16		;zero is flag for every day command
rcall	GrabTime
rcall	ToggleAC	;toggles relay

rjmp	LCCEnd

;------------------------------------

SC3:			;three touches grab day-of-week in addition

cpi		r21,3
breq	SC3a
rjmp	SC4


SC3a:

ldi		r16,1
sts		$6B,r16		;one equals add day of week information
rcall	GrabTime
rcall	ToggleAC

rjmp	LCCEnd


;-----------------------------

SC4:

cpi		r21,4
breq	SC4a
rjmp	SC5


SC4a:

ldi		r16,2		;two will step command every other day
sts		$6B,r16
rcall	GrabTime

rjmp	LCCEnd


;-------------------------------

SC5:

cpi		r21,5
breq	SC5a
rjmp	LCCEnd

SC5a:

ldi		r16,3		;sets flag for variable day command
sts		$6B,r16
rcall	GrabTime

rjmp	LCCEnd


;-----------------------------


LongCount:		;series for long counts


;---------------------------

LCC3:

cpi		r20,3		;Puts 17 commands into command memory, erasing old
				; commands.  Both summer and winter series can be
				; programmed.
				
breq	LCC3a
rjmp	LCC4


LCC3a:

ldi		r16,1		;set up for ONE green flash
rcall	FlashGLED	;indicate that one entry is needed

rcall	Button3Delay
cpi		r20,8		;if HC8 then proceed for summer times
breq	LCC3b

cpi		r20,9		;if HC9 then proceed for winter times
breq	LCC3c	
rjmp	LCCEnd


LCC3b:

rcall	OnTimes		;These are summer hours.  Starts at 8:00 p.m.
			; running until about 11:00 p.m.  On off cycle
			; is ten minutes for 17 cycles.
rjmp	LCCEnd


LCC3c:

rcall	OnTimes2	;These are winter hours.  Starts at 6:00 p.m.
			; running until about 11:00 p.m.  On off cycle
			; is seventeen minutes for 17 cycles.
rjmp	LCCEnd


;-----------------------------

LCC4:

cpi		r20,4		;Toggles GRAB TIME enable (SC2) or disable (SC1) modes
breq	LCC4a
rjmp	LCC5


LCC4a:

sbrs	r18,3
rjmp	LCC4b
cbr		r18,8
rjmp	LCCEnd


LCC4b:

sbr		r18,8
rjmp	LCCEnd


;------------------------------


LCC5:			;Toggle Display of Time or days away for 
			; every-other-day and variable day counters
cpi		r20,5	
breq	LCC5a
rjmp	LCC6


LCC5a:

sbrs	r18,5		;skip if bit 5(32) flag set
rjmp	LCC5b

cbr		r18,32		;clear bit 5(32) flag
rjmp	LCCEnd


LCC5b:

sbr		r18,32		;set bit 5(32) flag
rjmp	LCCEnd


;-----------------------

LCC6:			;0 = no AC outlet turn on at an intrustion 1= Yes, turn it on

cpi		r20,6		;Manually enters time into command memory
breq	LCC6a
rjmp	LCC7


LCC6a:

rcall	ManualProgramEntry

rjmp	LCCEnd


;------------------------

LCC7:				;Set clock

cpi		r20,7
breq	LCC7a
rjmp	LCC8


LCC7a:

rcall	SetClock
rjmp	LCCEnd


;------------------------------

LCC8:			;Steps every-other-day counter, flashing out
			; remaining days left before becoming active
			
cpi		r20,8	
breq	LCC8a
rjmp	LCC9			
			

LCC8a:

rcall	StepEveryOther		;steps every-other-day counter
cpi		r16,1			;check if one day off
breq	LCC8b
dec		r16
rcall	FlashRLED


LCC8b:

rcall	delay2310
rjmp	LCCEnd


;-------------------------------

LCC9:			;steps variable-day-counter.  Flashes out days left
			; before active day will occur
			;CAUTION, huge intervals will result in LONG displays
			
cpi		r20,9
breq	LCC9a
rjmp	LCC10


LCC9a:

rcall	StepVariable	;step variable day counter

cpi		r16,1
breq	LCC9b
dec		r16
rcall 	FlashRLED


LCC9b:

rcall	Delay2310
rjmp	LCCEnd


;---------------------------------

LCC10:			;Set LSB for compare register 3.58 countdown

cpi		r20,10
breq	LCC10a
rjmp	LCC11


LCC10a:

rcall	SetCompareRegister	;set 3.58 clock compare register during interrupts

rjmp	LCCEnd


;--------------------------------

LCC11:

cpi		r20,11		;Set number of seconds correction and how often
breq	LCC11a
rjmp	LCC12


LCC11a:

rcall	SecondsCorrection	;set correction
rjmp	LCCEnd

;------------------------------

LCC12:			;Enter new variable-day-counter value (1-255)

cpi		r20,12
breq	LCC12a
rjmp	LCC13


LCC12a:

rcall	Button3Delay

sts		$69,r21		;enter short count number into adjustable-day-count
sts		$6A,r21		
rjmp	LCCEnd


;------------------------------

LCC13:			;Clear command memory

cpi		r20,13
breq	LCC13a
rjmp	LCC14


LCC13a:

rcall	ZeroCommandMemory
rjmp	LCCEnd


;----------------------------

LCC14:				;READ DATA from EEPROM

cpi		r20,14
breq	LCC14a
rjmp	LCC15


LCC14a:

rcall	ReadEEPROM
rjmp	LCCEnd


;------------------------------

LCC15:				;WRITE DATA to EEPROM

cpi		r20,15
breq	LCC15a
rjmp	LCC16


LCC15a:

rcall	WriteEEPROM
rjmp	LCCEnd


;-----------------------------

LCC16:

rjmp	LCCEnd


LCCEnd:

rcall	TwoWags
rcall	Delay165

pop		r16

ret


;---------------------------------------


ClockCntDwnInt: 		;interrupt to update clock counters

push	r16

in		r16,SREG	;save flags
sts		$60,r16

sbr		r18,4		;set bits 2 (4) -- second flag 
			;(set each second)

inc		r15			;step one count of SECONDS counter
mov		r16,r15

cpi		r16,60
breq	CCDI1

rjmp	CCDIExit		;return since no action required


CCDI1:

clr		r15		;zero r15 -- SECONDS counter

sbr		r18,16		;set bit 4 (16) in R18 -- the NEW MINUTE flag


CCDI1Cntr:

lds		r16,$61		;load low minutes byte into R16

inc		r16
sts		$61,r16

brne	CCDI1a		;jump out if not zero (no overflow)

lds		r16,$62		;high byte minutes
inc		r16			;increment high byte minutes
sts		$62,r16


CCDI1a:

lds		r16,$62		;check high byte if it is 5
cpi		r16,5

breq	CCDI1b

rjmp	CCDIalarmExit


CCDI1b:			

lds		r16,$61
cpi		r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIalarmExit


CCDI1c:			;zero high and low bytes - it is midnight

clr		r16
sts		$61,r16
sts		$62,r16

lds		r16,$63
inc		r16
sts		$63,r16

cpi		r16,8

breq	CCDI1d


CCDI1e:

rcall	ClockCorrection	;call clock correction (midnight corr)
rcall	StepEveryOther		;steps every-other-day counter
rcall	StepVariable		;steps variable-day-counter


rjmp	CCDIalarmExit


CCDI1d:

ldi		r16,1		;load with 1 (Sunday)
sts		$63,r16
rjmp	CCDI1e		;to continue


CCDIalarmExit:

rcall	CheckAllAlarms	



CCDIExit:		;Exit this interrupt routine

lds		r16,$60		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r16

reti


;-----------------------------


CheckAllAlarms:		;check entire list of alarm commands for match

push	r14		;temporary x register store
push	r25
push	r26
push	r28
push	r29
push	r30
push	r31


ldi		r26,$6C		;low byte -- start of alarm memory - x register
mov		r14,r26

CAAc:			;check for end of list

mov		r26,r14		;restore x register value to r26
lds		r16,$67		;get present bookmark for command memory pointer
cp		r26,r16
brlo	CAAa
rjmp	CAAEnd		;jump out of loop if equal or higher (end of list)

CAAa:			;Is there a time match?

ld		r30,x+		;load low command byte into r30 and post decrement
ld		r31,x+		;load high command byte into r31 and post decrement
mov		r14,r26		;save x register location in r14
mov		r25,r31
andi	r25,248		;save high bytes for later processing in r25

lds		r28,$61		;get minutes low and put in r28
lds		r29,$62		;get minutes high and put in r29

andi	r31,7		;retain only low bits 0,1,2 of high byte in r31

cp		r28,r30		;low byte compare
breq	CAAa1		;do further comparing

rjmp	CAAc		;go for another round of search


CAAa1:

cp		r29,r31		;high byte compare
breq	CAAb

rjmp	CAAc		;go for another round of search


CAAb:			;do further checks on high bytes of MSD

tst		r25		;check for command bit in r25 for all being zero
			; (every day command)
breq	CAAf		;toggle relay


CAAg:

cpi		r25,64		;check for every-other-day conditiion
breq	CAAi		;it is an every other day type command


CAAh:			;check for variabke day match

cpi		r25,128		;check for variable day match
breq	CAAj


CAAk:			;it's a day-of-week command

lsr		r25
lsr		r25
lsr		r25		;shift day of week into position

lds		r16,$63
cp		r25,r16
breq	CAAf		;toggle relay
rjmp	CAAc		;go for another round


CAAi:

lds		r16,$68

cpi		r16,1		;is it ONE?  If it is, execute command
breq	CAAf		;toggle relay
rjmp	CAAc		;go for another round


CAAj:

lds		r16,$6A		;test for one - toggles relay when one
cpi		r16,1
breq	CAAf		;it is a match -- toggle relay
rjmp	CAAc


CAAf:			;toggle relay and return to next check

rcall	ToggleAC	;toggle relay
rjmp	CAAc


CAAEnd:

pop		r31
pop		r30
pop		r29
pop		r28
pop		r26
pop		r25
pop		r14

ret

;-------------------------------------------------------


ClockCorrection:	;subtracts seconds in $66 to time at minight 
			
lds		r16,$66
tst		r16		;if zero, no correction is needed
			;1= every day, 2= every other day, etc.

breq	CCExit		;exit routine with no action

dec		R16
sts		$66,r16		;store new value in variable

breq	CC3		;it is time for update of second counter

ret


CC3:

lds		r16,$64		;seconds correction from permanent store
mov		r15,r16		;put in seconds store area to correct time

lds		r16,$65		;get permanently stored value for interval correction
sts		$66,r16		;store new value in variable


CCExit:

ret

;----------------------------------------------


VarClockFlash:		;flashes red or green LED (G=AM R=PM)
			;R16 contains number of flashes
push	r17

rcall	Idelay2310	;delay before flashes start

lds		r17,$62		;check if AM or PM time (less than 720 = AM)
cpi		r17,2		;go to AM if high byte is <2 
brlo	VCF1		;jump to AM routine
cpi		r17,2
breq	VCF3
rjmp	VCF2		;jump to PM routine


VCF3:

lds		r17,$61		;compare low byte for less than 208
cpi		r17,208		
brlo	VCF1		;jump to AM routine
rjmp	VCF2		;jump to PM routine


VCF1:			; AM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

sbi		PortB,1		;turn on GREEN LED (AM)
rcall	IDelay330	;on time
cbi		PortB,1		;off Green LED

sbrc	r18,1		;skip if bit 1 is clear - no double flash
rjmp	VCF1ab		;go for double flash


VCF1ac:

rcall	IDelay495		;off time
dec		r16
rjmp	VCF1


VCF1ab:			;gives double flash

rcall	IDelay165	;small pause
sbi		PortB,1		;on green LED
rcall	IDelay330	;on time
cbi		PortB,1		;off Green LED
rjmp	VCF1ac


VCF2:			; PM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

sbi		PortB,0		;turn on RED LED
rcall	IDelay330	;on time delay
cbi		PortB,0		;turn off RED LED
sbrc	r18,1		;skip if bit 1 is clear - no double flash
rjmp	VCF2ab


VCF2ac:

rcall	IDelay495		;off time delay
dec		r16
rjmp	VCF2


VCF2ab:			;give double red flash

rcall	IDelay165	;short off delay
sbi		PortB,0		;on Red LED
rcall	IDelay330	;on time delay for second flash
cbi		PortB,0		;off Red LED
rjmp	VCF2ac


VCF2a:

pop		r17
cbr		r18,2		;clear bit 1 (2) - double flash flag
ret

;-------------------------------------------


DisplayClock:		;displays time

clr		r6		;zero hours count
clr		r7		;zero MSD of minutes count

bclr	7		;turn off interrupts

lds		r28,$61		;load low byte into r28
lds		r29,$62		;load high byte into r29

bset	7		;turn interrupts back on

tst		r29		;test high byte for zero
brne	DC3

cpi		r28,60		;check to see if less than 60 (12:00 am to 12:59 am)
brlo	DC2
rjmp	DC3


DC2:

ldi		r16,12		;flash out 12 a.m.

rcall	VarClockFlash	;flash out time of 12 a.m.

rjmp	DC7


DC3:			;flash out other times

mov		r4,r28		;save low byte in r4
mov		r5,r29		;save high byte in r5

inc		r6		;increment hours count
sbiw	r28,60		;subtract 60 from the r28,29 pair

brcs	DC3a		;hours now almost decoded

rjmp	DC3		;repeat loop again


DC3a:

dec		r6		;restore old value of hours
mov		r28,r4		;restore low byte in r28
			;at this point, minutes remainder is in low byte
			; (r29 will be zero)
			

DC10:

mov		r16,r6		;put hours into r16 for check
cpi		r16,13
brsh	DC9
rjmp	DC8


DC9:

subi	r16,12


DC8:			;flash out time

rcall	VarClockFlash


DC7:			;decode minutes with MSD double flashed

mov		r16,r28
inc		r7		;use r7 for MSD count
subi	r28,10		;get remaining minutes
brcs	DC7a

rjmp	DC7		;loop back for another subtraction


DC7a:

dec		r7		;restore correct value for MSD
mov		r28,r16		;restore correct remainder (LSDs)

mov		r16,r7		;put in MSD for flashout
sbr		r18,2		;set bit 1 (2) for double flash
rcall	VarClockFlash	;flash out MSD
cbr		r18,2		;clear bit 1 (2) of double flash

mov		r16,r28
rcall	VarClockFlash	;flash out LSDs

ret


;---------------------------


ZeroCommandMemory:	;zero all command storage locations

push 	r1
		
clr		r1
ldi		r26,$6C	;low start byte for start of storage locations
sts		$67,r26	;store new bookmark for start of entries

ldi		r16,68	;for zeroing 64 locations

ZCM:

st		x+,r1	;zero location

dec		r16	
brne	ZCM

pop		r1
ret

;----------------------------------


ToggleAC:			;toggles AC power on/off

sbrs	r18,6		;skip if r18,6(64) is set -- relay state is on
rjmp	TAC1

sbi		PortB,4		;on bit 4 for off pulse
rcall	Delay40th
cbi		PortB,4 	;off bit 4 - end of pulse
cbr		r18,64		;relay state off, r18,6(64)
ret

TAC1:

sbi		PortB,3		;set bit 3 for on pulse
rcall	Delay40th
cbi		PortB,3		;off bit 3 - end of pulse
sbr		r18,64		;relay state to on, r18,6(64)
ret


;--------------------------

SetCompareRegister:

		;puts new value into compare register A.  
		;Default = 218 for high byte, 140 for low byte.
		
push	r1

clr		r1
rcall	TwoGreenFlash
rcall	OneGreenFlash	;total of three flashes indicating multiple inputs

rcall	Button3Delay
rcall	Mult100		;multiply r20 value by 100
mov		r1,r20		;set 100's column

rcall	OneGreenFlash	;indicate an entry is needed
rcall	Button3Delay
rcall	Mult10		;multiply r20 value by 10
add		r1,r20		;add in tens units

rcall	OneGreenFlash	;indicate an entry is needed
rcall	Button3Delay
add		r1,r20		;add in LSD of count -- data is now complete

ldi		r16,218
out 	OCR1AH,r16
out		OCR1AL,r1	;put new compare register A data in place

pop		r1
ret

;-------------------------------------

SecondsCorrection:	;Set seconds of correction and how often (2 entries)

rcall	TwoGreenFlash
rcall	Button3Delay
sts		$64,r20		;seconds of correction (zero to 255)

rcall	OneGreenFlash
rcall	Button3Delay	;enter interval from 0 to 255 - zero = no correction
sts		$65,r20
sts		$66,r20			;put value both in perm and temp store areas

ret

;---------------------

SetClock:		;sets clock for AM/PM, hour, minutes, and day-of-week
			;  24 hour time entry required

push	r1		;minutes accumulator
push	r2		;hours accumulator
push	r3		;day of week accumulator
push	r26
push	r27

clr		r26		;low byte accumulator
clr		r27		;high byte accumulator

ldi		r16,5		;set up for FIVE green flashes
rcall	FlashGLED	;indicate that five entries are needed

rcall	Button3Delay
mov		r3,r20		;day of week (1=Sunday, 7=Saturday)

rcall	OneGreenFlash
rcall	Button3Delay	;enter MSD of hours 

rcall	Mult10		;multiply value by 10 and store in r2
mov		r2,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	Button3Delay	;enter LSD of hours
add		r2,r20		;value in hours now complete in r2
	
rcall	TwoGreenFlash	;signal another entry needed
rcall	Button3Delay	;enter msd of minutes
rcall	Mult10		;multiply by 10 and store in r1
mov		r1,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	Button3Delay
add		r1,r20		;value in minutes now complete


SetC1:

tst		r2		;total hours count
breq	SetC2		;done
adiw	r26,60
dec		r2
rjmp	SetC1


SetC2:

add		r26,r1		;total minutes count
brcs	SetC3
rjmp	SetEnd


SetC3:

inc		r27		;carry flag was set
rjmp	SetEnd


SetEnd:

bclr	7		;turn off global interrupts

sts		$63,r3		;store day-of-week
sts		$61,r26		;store low byte in $61
sts		$62,r27		;store high byte in $62
clr		r15		;zero seconds counter

bset	7		;turn on global interrupts

pop		r27
pop		r26
pop		r3
pop		r2
pop		r1

ret


;--------------------------------------

;FLASHRLED - flashes Red LED No. of Times in r16 (non-interruptible)

FlashRLED:

push	r0
mov		r0,r16

	
FlashRLEDa:

tst		r16
breq	FlashRLEDend
						;put value of R16 in R0
sbi		PORTB,0			;Turn on RED LED
rcall	Delay165
cbi		PORTB,0			;Off RED LED
rcall	Delay1155	
dec		r0
brne	FlashRLEDa


FlashRLEDend:

pop		r0

ret

;-----------------------

FlashGLED:		;Flash Green LED no. of time in R16 (non-interruptible)

push	r0
mov		r0,r16

tst		r16
breq	FlashGLEDend


FlashGLEDa:

rcall	Delay495
sbi		PORTB,1			;Turn on GREEN LED
rcall	Delay165
cbi		PORTB,1			;Off GREEN LED
dec		r0
brne	FlashGLEDa


FlashGLEDend:

pop		r0

ret

;------------------------------------

OneRedFlash:

ldi		r16,1
rcall	FlashRLED
ret

;------------------------------------

OneGreenFlash:

ldi		r16,1
rcall	FlashGLED
ret

;------------------------------------


TwoGreenFlash:

ldi		R16,2		;prepare for two flashes
rcall	FlashGLED
ret

;-------------------------------------


TwoRedFlash:

ldi		R16,2		;prepare for two flashes
rcall	FlashRLED	
ret

;------------------------------------

RapidRedFlashInterruptible:	;r16 contains number of rapid red flashes

RRF1:

sbi		PortB,0		;on red LED
rcall	IDelay165
cbi		PortB,0		;off red led
rcall	IDelay495

dec		r16
brne	RRF1

ret	


;-------------------------------------


IGFlash:			;Interruptible green flash - R16  contains 
				; number of flashes

tst		r16
breq	IGF1
sbi		PortB,1		;on Green LED
rcall	IDelay330
cbi		PortB,1		;off Green LED
rcall	IDelay1155
dec		r16
rjmp	IGFlash


IGF1:
rcall	IDelay2310
ret				

;----------------------------


IRFlash:			;Interruptible red flash - R16  contains 
				; number of flashes

tst		r16
breq	IRF1
sbi		PortB,0		;on Green LED
rcall	IDelay330
cbi		PortB,0		;off Green LED
rcall	IDelay1155
dec		r16
rjmp	IRFlash


IRF1:
rcall	IDelay2310
ret				


;---------------------------


VarWagRGInterruptible:		;R16 contains number of wags

sbi		PortB,0
rcall	IDelay165
cbi		PortB,0
rcall	IDelay165
sbi		PortB,1
rcall	IDelay165
cbi		PortB,1
rcall	IDelay165
dec		r16
brne	VarWagRGInterruptible

ret

;-------------------------------

VarWagRG:		;R16 contains number of wags

sbi		PortB,0
rcall	Delay40th
cbi		PortB,0
rcall	Delay40th
sbi		PortB,1
rcall	Delay40th
cbi		PortB,1
rcall	Delay40th
dec		r16
brne	VarWagRG

ret

;-----------------------

TwoWags:		;do two wags of the VarWagRG

ldi		r16,2

rcall	VarWagRG
ret

;-----------------------

;Delay1 subroutines

;-------------------------

Delay40th:			; 1/40th second delay

push	r31
push	r0

ldi		r31,116		; high byte
clr		r0


DelayA:

dec		r0
brne	DelayA

dec		r31
brne	DelayA

pop		r0
pop		r31

ret


;--------------------------

Delay2310:

ldi		r31,91
rjmp		DelayStart


Delay1155:		;Delay series with r31 containing zero value at start
			; r31 is a reserved variable
ldi		r31,45
rjmp	DelayStart


Delay495:

ldi		r31,20
rjmp	DelayStart


Delay165:

ldi		r31,6
rjmp	DelayStart


DelayStart:

rcall	Delay40th
dec		r31
brne	DelayStart
ret


;------------------------

IDelay2310:		;interruptible delay routines

ldi		r31,61		; old 24 divided by .390625
rjmp	IDelayStart


IDelay1155:

ldi		r31,31
rjmp	IDelayStart


IDelay495:

ldi		r31,15
rjmp	IDelayStart


IDelay330:

ldi		r31,5
rjmp	IDelayStart


IDelay165:

ldi		r31,1
rjmp	IDelayStart


IDelayStart:

ldi		r24,100		;first delay countdown
push	r1

clr		r1			;zero	


IDelayA:

dec		R24
brne	IDelayA


IDelayA1:

ldi		r24,100		;restore initial countdown number
push	r16


IDS5:

sbis	PIND,0x00	;Skip if button not pushed
rjmp	IDelayB
pop		r16

ldi		r24,100		;Restore first count number
dec		R1
brne	IDelayA

dec		r31	
brne	IDelayA
rjmp	IDelayC


IDelayB:

cbi		PortB,0		;off red and green LEDs
cbi		PortB,1

rcall	DetectCntTwo	;new function service is required from button
pop		r16


IDelayC:
	
pop		r1

ret

;--------------------------


Mult100:		;Multiplies value in R20 by 100 and returns value in R20

push	r1
clr		r1	;zero r1 for upcoming data

tst		r20
breq	Mult100b

ldi		r16,100	;the multiplication value


Mult100a:

add		r1,r16
dec		r20
brne	Mult100a


Mult100b:

mov		r20,r1	;for return of value
pop		r1
ret


;--------------------------------------

Mult10:		; multiplies value in R20 by 10 returning value in R20

push 	r1
clr		r1		;zero R1

tst		R20		;test long count value
breq	Mult10b		;value is zero

ldi		r16,10		;put ten into r16


Mult10a:

add		r1,r16		;add ten for each value in R1
dec		r20
brne	Mult10a


Mult10b:

mov		r20,r1		;move r1 value into r16
pop		r1
ret

;-----------------------------

StepEveryOther:		;steps every-other-day counter

lds		r16,$68		;every-other-day counter
dec		r16		;step the counter
breq	SEOa
rjmp	SEOb


SEOa:

ldi		r16,2


SEOb:

sts		$68,r16
ret

;-----------------------------

StepVariable:		;steps variable-day counter

lds		r16,$6A

dec		r16

breq	SVa 		
rjmp	SVb


SVa:

lds		r16,$69


SVb:

sts		$6A,r16		;store new value into $6A variable
ret
			

;------------------------------

GrabTime:		;grabs and stores time into command memory

push	r25		;scatch
push	r26		;pointer

lds		r26,$67		;get bookmark for command memory pointer
			; put in "X" register

cpi		r26,$B0		;check for end of memory storage
brsh	GTEnd1		;exit if count is same or higher


bclr	7		;disable global interrupts

lds		r16,$61		;grab low time byte
st		x+,r16		;store in command memory
lds		r16,$62		;grab high time byte

lds		r25,$63		;grab day of week
lsl		r25
lsl		r25
lsl		r25		;move day of week into position

or		r16,r25		;add day of week info into r16
st		x+,r16		;store in command memory

bset	7		;enable global interrupts

sts		$67,r26		;store away bookmark

lds		r25,$6B		;get value of GrabTime Flag

cpi		r25,0		
breq	GT1

cpi		r25,1
breq	GT2

cpi		r25,2
breq	GT3


GT4:			;last choice - insert variable day command

ld		r16,-x	;pre decrement x and grab high byte
andi	r16,7	;save only bits 0,1,2
ori		r16,128	;set bit 7 (128) for variable day command
st		x+,r16	;put back in memory and increment x register
rjmp	GTEnd


GTEnd1:		;long red LED flash if no more memory available

sbi		PORTB,0		;on LED
rcall	Delay2310	
cbi		PORTB,0		;off LED


GTEnd:

pop		r26
pop		r25
ret


GT1:		;It is an every day command
		
ld		r16,-x	;pre decrement x and grab high byte
andi	r16,7	;mask out day of week information - save bits 0,1,2 only
st		x+,r16	;put value back into command memory and increment x register

rjmp	GTEnd


GT2:		;it is on a given day-of-week command

rjmp	GTEnd	;leave unchanged since day of week info already there


GT3:		;it is an every-other-day command

ld		r16,-x	;pre decrement x and grab high byte
andi	r16,7	;save only bits 0,1,2
ori		r16,64	;set bit 6 (64) for every-other-day command
st		x+,r16	;store back and increment x register

rjmp	GTEnd


;--------------------------------------------

DisplayDayCountdown:	;Displays the day countdowns for both the every-other-
			;  day and variable day countdowns
			
sbi		PORTB,1		;turn on green led
rcall	idelay2310	;long green on time
cbi		PORTB,1		;off led

rcall	idelay1155
lds		r16,$68		;get present every other day value
dec		r16
rcall	IGFlash		;flash green number of days -1

rcall	Idelay1155	;pause

sbi		PORTB,0		;turn on red led
rcall	Idelay2310	;long red on time
cbi		PORTB,0		;off led

rcall	Idelay1155
lds		r16,$6A		;get present variable day value
dec		r16
rcall	IRFlash		;flash out number of variable days -1

rcall	Idelay1155

ret

;--------------------------------------------

OnTimes:		;sets a series of on times spread 10 minutes apart
			; starting at 8:00 p.m. -- SUMMER series

rcall	ZeroCommandMemory	;clear all commands

push	r26		
push	r28
push	r29

ldi		r26,$6C		;start of command memory store area
ldi		r16,10		;for 10 storage locations

ldi		r29,4		;start at 8:00 p.m.
ldi		r28,176	


TOTa:

st		x+,r28
st		x+,r29

adiw	r28,19		;add 19 minutes on/off time to word r28,r29

dec		r16

brne	TOTa

rjmp	TOT2b		;to finish and ret

;-------------------------------

OnTimes2:		;sets a series of on times spread 17 minutes apart
			; starting at 6:00 p.m.-- WINTER series

rcall	ZeroCommandMemory	;clear all commands

push	r26		
push	r28
push	r29

ldi		r26,$6C		;start of command memory store area
ldi		r16,10		;for 10 storage locations

ldi		r29,4		;start at 6:00 p.m.
ldi		r28,56	


TOT2a:

st		x+,r28
st		x+,r29

adiw	r28,29		;add 29 minutes to word r28,r29

dec		r16

brne	TOT2a


TOT2b:

sts		$67,r26		;store bookmark

pop		r29
pop		r28
pop		r26

ret

;-------------------------------

ManualProgramEntry:

			;manually enters time commands into command store

push	r25		;scratch
push	r26		;push X register
push	r27		
push	r28


MPEs:

lds		r26,$67		;get bookmark for command memory pointer
			; put in "X" register

cpi		r26,$B0		;check for end of memory storage
brsh	MPE1		;exit if count is same or higher
rjmp	MPE2		;entry okay, ... continue


MPE1:			;give long RED flash showing memory is full

sbi		PORTB,0		;on LED
rcall	Delay2310	
cbi		PORTB,0		;off LED

rjmp	MPEEnd



MPE2:

ldi		r16,6			;set up for SIX green flashes
rcall	FlashGLED		;indicate that six entries are needed

rcall	Button3Delay	;Enter MSD hours value
rcall	Mult10			;multiply r20 value by TEN for MSD

mov		r25,r20			;save r20 MSD value in r25

rcall	OneGreenFlash	;flash Green LED to indicate another entry needed

rcall	Button3Delay	;enter LSD hours value
add		r25,r20			;add LSD hours to r25 value

clr		r26
clr		r27		;zero r26 and r27 (X register)


MPE2a:

tst		r25
breq	MPE2b

adiw	r26,60		;add value of 60 to x reigster pair
dec		r25	
rjmp	MPE2a


MPE2b:			;hours converted to minutes at this point

rcall	OneGreenFlash	;flash Green LED to indicate another entry needed

rcall	Button3Delay	;get MSD of minutes
rcall	Mult10			;multiply value by 10
add		r26,r20			;add MSD X 10 to r26
brcs	MPE2b1			;branch if carry is set
rjmp	MPE2b2


MPE2b1:

inc		r27		;add one to highest byte of x register pair


MPE2b2:


rcall	OneGreenFlash	;flash green LED indicating another entry is needed

rcall	Button3Delay	;get LSD of minutes
add		r26,r20			;add LSD to r26
brcs	MPE2b3			;branch if carry is set
rjmp	MPE2b4	


MPE2b3:

inc		r27		;add one to MS byte of x register pair	


MPE2b4:					;correct value in minutes in x register pair

andi	r27,7			;save only lowest time digits in event of entry error

rcall	OneGreenFlash	;indicate another entry is needed

rcall	Button3Delay	;get Day-of-week or other command entry
cpi		r20,8
brsh	MPE5			;branch is same or higher for other process


MPE4:

lsl		r20
lsl		r20
lsl		r20		;move day-of-week or every day (0) command into position

or		r27,r20		;add data to MS byte of x register pair


MPE8:		;add data to command memory store and increment 
			;  bookmark pointer

bclr	7		;off interrupts

lds		r28,$67		;grab bookmark and put in y register

st		y+,r26
st		y+,r27		;store values into command store area

sts		$67,r28		;save bookmark

bset	7		;turn interrupts back on



MPE9:

rcall	OneGreenFlash	;indicate another entry is needed

rcall	Button3Delay	;enter whether to continue (0) or exit (1 or higher)

cpi		r20,0			;go for another if zero
breq	MPEs			;go for another time entry
rjmp	MPEEnd


MPE5:

cpi		r20,8		
breq	MPE7		;it is every-other-day command


MPE6:			;it is a variable day command

ori		r27,128
rjmp	MPE8		;set bit for variable day command


MPE7:			;it is every-other-day command

ori		r27,64		;set bit for every-other-day
rjmp	MPE8



MPEEnd:

pop		r28
pop		r27
pop		r26
pop		r25

ret

;----------------------------------------

ReadEEPROM:


bclr	7			;off global interuupts

push	r26			;"X" register

ldi		r16,0			;start of EEPROM data
rcall	ReadEP
inc		r16			;got to next address location
sts		$67,R19		;put bookmark into SRAM
ldi		r26,$6C		;get "X" register set up at beginning of SRAM



ReadE1:

rcall	ReadEP		;get new EEPROM byte for store in SRAM
st		x+,r19		;store and post increment X register
inc		r16			;go to next location in EEPROM
cpi		r16,70
brsh	ReadEnd

rjmp	ReadE1		;repeat loop


ReadEnd:

pop		r26
bset	7			;On global interrupts
ret


;-----------------------------------------

WriteEEPROM:	;R16 address in, R19 data out

bclr	7		;Off global interrupts

push	r26		;X register
lds		r19,$67	;Store bookmark
ldi		r16,0
rcall	WriteEP
inc		r16
ldi		r26,$6C	;start of SRAM


WriteE1:

ld		r19,x+	;load SRAM byte
rcall	WriteEP
inc		r16
cpi		r16,70
brsh	WriteEEnd
rjmp	WriteE1


WriteEEnd:

pop		r26
bset	7		;On global interrupts
ret


;----------------------------------------

ReadEP:				;R16 data in, R19 data out


sbic	EECR,EEWE
rjmp	ReadEP

out		EEAR,r16
sbi		EECR,EERE

in		r19,EEDR
ret


;-------------------------------------

WriteEP:			;r16 = address, r19 = data

sbic	EECR,EEPE
rjmp	WriteEP

out		EEAR,r16	;out address

out		EEDR,r19	;out data
sbi		EECR,EEMPE	;out one to EEMPE
sbi		EECR,EEPE

ret


;---------------------------

